//
//  Database Engine Wrapper
//  Version 1.0
//  CURSOR.H
//  Copyright  1994
//  B. W. Bush
//  All rights reserved
//


#ifndef BWB_DEW_CURSOR
#define BWB_DEW_CURSOR


// TDEWCursor class.

class TDEWCursor
  : public MDEWObject
{
    friend class TDEWRecord;

  public:

    TDEWCursor();
    TDEWCursor(const hDBICur);
    TDEWCursor(TDEWDatabase&);
    TDEWCursor(TDEWDatabase&, const pcCHAR, const pcCHAR = NULL, const DBIOpenMode = dbiREADWRITE, const DBIShareMode = dbiOPENSHARED, const pcCHAR = NULL, const pcCHAR = NULL, const UINT16 = NULL, const XLTMode = xltFIELD, const BOOL = FALSE, const pcBYTE = NULL);
    TDEWCursor(const TDEWCursor&, const BOOL = FALSE, const BOOL = FALSE);
    ~TDEWCursor();

    TDEWCursor& operator=(const TDEWCursor&);

    void AcqTableLock(const DBILockType = dbiREADLOCK);
    void AddIndex(const IDXDesc&, const pcCHAR = NULL);
    BOOL Advance(const DBILockType = dbiNOLOCK, const pRECProps = NULL);
    void AppendRecord();
    void BeginLinkMode();
    void Close();
    void CloseIndex(const pcCHAR, const UINT16 = NULL);
    CMPBkMkRslt CompareBookMarks(const pcBYTE, const pcBYTE) const;
    /*CURProps& CursorProps();*/
    const CURProps& CursorProps() const;
    hDBIDb DatabaseHandle() const {return fHDatabase;}
    void DeleteIndex(const pcCHAR, const UINT16, const pcCHAR);
    TDEWRecord& DeleteRecord();
    void EmptyTable();
    void EndLinkMode();
    /*pFLDDesc FieldDescs();*/
    pcFLDDesc FieldDescs() const;
    const TDEWRecord& ForceReread() const;
    void GetBookMark(const pBYTE) const;
    void GetCallBack(const CBType, UINT32&, UINT16&, pVOID&, pfDBICallBack&) const;
    void GetIndexDesc(const UINT16, IDXDesc&) const;
    void GetIndexDesc(IDXDesc&) const;
    void GetIndexForField(const UINT16, IDXDesc&, const BOOL = FALSE) const;
    TDEWRecord& GetNextRecord(const DBILockType = dbiNOLOCK, const pRECProps = NULL);
    TDEWRecord& GetPriorRecord(const DBILockType = dbiNOLOCK, const pRECProps = NULL);
    TDEWRecord& GetRecord(const DBILockType = dbiNOLOCK, const pRECProps = NULL);
    TDEWRecord& GetRecordForKey(const UINT16 iFields = NULL, const UINT16 iLen = NULL) {return GetRecordForKey(*fRecord, iFields, iLen);}
    TDEWRecord& GetRecordForKey(const TDEWRecord&, const UINT16 = NULL, const UINT16 = NULL);
    TDEWRecord& GetRelativeRecord(const INT32, const DBILockType = dbiNOLOCK, const pRECProps = NULL);
    void GetRintDesc(const UINT16, RINTDesc&) const;
    void GetVchkDesc(const UINT16, VCHKDesc&) const;
    hDBICur Handle() const {return fHCursor;}
    UINT16 IndexSeqNo(const pcCHAR, const UINT16 = NULL, const pcCHAR = NULL) const;
    TDEWRecord& InitRecord();
    TDEWRecord& InsertRecord(const DBILockType = dbiNOLOCK);
    BOOL IsRecordLocked() const;
    UINT16 IsTableLocked(const DBILockType = dbiREADLOCK) const;
    BOOL IsTableShared() const;
    void LinkDetail(TDEWCursor&, const UINT16, const pcUINT16, const pcUINT16);
    void LinkDetail(TDEWCursor&, const UINT16, const pcCHAR);
    TDEWCursor LockList(const BOOL = TRUE, const BOOL = TRUE) const;
    void MakePermanent(const pcCHAR, const BOOL = TRUE);
    TDEWRecord& ModifyRecord(const BOOL = TRUE);
    void Open(TDEWDatabase&, const pcCHAR, const pcCHAR = NULL, const DBIOpenMode = dbiREADWRITE, const DBIShareMode = dbiOPENSHARED, const pcCHAR = NULL, const pcCHAR = NULL, const UINT16 = NULL, const XLTMode = xltFIELD, const BOOL = FALSE, const pcBYTE = NULL);
    void Open(const pcCHAR, const pcCHAR = NULL, const DBIOpenMode = dbiREADWRITE, const DBIShareMode = dbiOPENSHARED, const pcCHAR = NULL, const pcCHAR = NULL, const UINT16 = NULL, const XLTMode = xltFIELD, const BOOL = FALSE, const pcBYTE = NULL);
    void OpenIndex(const pcCHAR, const UINT16 = NULL);
    void PackTable(const BOOL = TRUE);
    TDEWRecord& Record();
    const TDEWRecord& Record() const;
    UINT32 RecordCount() const;
    void RegenIndex(const pcCHAR, const UINT16 = NULL, const pcCHAR = NULL);
    void RegenIndexes();
    void RegisterCallBack(const CBType, const UINT32, const UINT16, const pVOID, const pfDBICallBack);
    void RelRecordLock(const BOOL = TRUE);
    void RelTableLock(const DBILockType = dbiREADLOCK, const BOOL = TRUE);
    void SaveChanges() const;
    UINT32 SeqNo() const;
    void SetFieldMap(const UINT16, const pcFLDDesc);
    TDEWRecord& SetToBegin();
    TDEWRecord& SetToBookMark(const pcBYTE);
    TDEWRecord& SetToCursor(const TDEWCursor&);
    TDEWRecord& SetToEnd();
    TDEWRecord& SetToKey(const DBISearchCond eSearchCond = keySEARCHEQ, const UINT16 iFields = NULL, const UINT16 iLen = NULL) {return SetToKey(*fRecord, eSearchCond, iFields, iLen);}
    TDEWRecord& SetToKey(const TDEWRecord&, const DBISearchCond = keySEARCHEQ, const UINT16 = NULL, const UINT16 = NULL);
    TDEWRecord& SetToRecordNo(const UINT32);
    TDEWRecord& SetToSeqNo(const UINT32);
    void Sort(const UINT16, const pcUINT16, const pcCHAR = NULL, const pcBOOL = NULL, const pcSORTOrder = NULL, pfSORTCompFn* = NULL, const BOOL = FALSE);
    void SwitchToIndex(const pcCHAR, const BOOL = TRUE, const UINT16 = NULL, const pcCHAR = NULL);
    TDEWRecord& UndeleteRecord();
    void UnlinkDetail();

  private:

    void GetProperties();

    CURProps fCursorProps;
    pFLDDesc fFieldDesc;
    hDBICur fHCursor;
    hDBIDb fHDatabase;
    TDEWRecord* fRecord;
};


#endif // BWB_DEW_CURSOR
