//
//  Database Engine Wrapper
//  Version 1.0
//  ENGINE.CPP
//  Copyright  1994
//  B. W. Bush
//  All rights reserved
//


// Definitions.

#include <dew.h>        // database engine wrapper


// Constructor.

TDEWEngine::TDEWEngine(const pcDBIEnv pEnv)
{
    Open(pEnv);
}


// Destructor.

TDEWEngine::~TDEWEngine()
{
    Close();
}


// Add a password to current session.

void TDEWEngine::AddPassword(const pcCHAR pszPassword)
{
    TDEWException::CheckResult(DbiAddPassword((pCHAR) pszPassword));
}


// Open a cursor on "config".

TDEWCursor TDEWEngine::CfgInfoList(const pcCHAR pszCfgPath, const DBIOpenMode eOpenMode, const CFGMode eConfigMode) const
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenCfgInfoList(NULL, eOpenMode, eConfigMode, (pCHAR) pszCfgPath, &hCursor));

    return TDEWCursor(hCursor);
}


// Check refresh for session.

void TDEWEngine::CheckRefresh() const
{
    TDEWException::CheckResult(DbiCheckRefresh());
}


// Close engine.

void TDEWEngine::Close()
{
    TDEWException::CheckResult(DbiExit());
}


// Get a list of registered databases.

TDEWCursor TDEWEngine::DatabaseList() const
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenDatabaseList(&hCursor));

    return TDEWCursor(hCursor);
}


// Get current date format.

const FMTDate& TDEWEngine::DateFormat() const
{
    return fFmtDate;
}


// Set debug layer options.

void TDEWEngine::DebugLayerOptions(const UINT16 iOption, const pcCHAR pszDebugFile)
{
    TDEWException::CheckResult(DbiDebugLayerOptions(iOption, (pCHAR) pszDebugFile));
}


// Get a list of driver names.

TDEWCursor TDEWEngine::DriverList() const
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenDriverList(&hCursor));

    return TDEWCursor(hCursor);
}


// Drop a password from current session.

void TDEWEngine::DropPassword(const pcCHAR pszPassword)
{
    TDEWException::CheckResult(DbiDropPassword((pCHAR) pszPassword));
}


// Get a list of field types.

TDEWCursor TDEWEngine::FieldTypesList(const pcCHAR pszDriverType, const pcCHAR pszTblType) const
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenFieldTypesList((pCHAR) pszDriverType, (pCHAR) pszTblType, &hCursor));

    return TDEWCursor(hCursor);
}


// Get callback function.

void TDEWEngine::GetCallBack(const CBType ecbType, UINT32& iClientData, UINT16& iCbBufLen, pVOID& pCbBuf, pfDBICallBack& pfCb) const
{
    TDEWException::CheckResult(DbiGetCallBack(NULL, ecbType, &iClientData, &iCbBufLen, &pCbBuf, &pfCb));
}


// Get client info.

void TDEWEngine::GetClientInfo(CLIENTInfo& clientInfo) const
{
    TDEWException::CheckResult(DbiGetClientInfo(&clientInfo));
}


// Get a description of a logical db.

void TDEWEngine::GetDatabaseDesc(const pcCHAR pszName, DBDesc& dbDesc) const
{
    TDEWException::CheckResult(DbiGetDatabaseDesc((pCHAR) pszName, &dbDesc));
}


// Get description for a given type.

void TDEWEngine::GetDriverDesc(const pcCHAR pszDrvType, DRVType& drvType) const
{
    TDEWException::CheckResult(DbiGetDriverDesc((pCHAR) pszDrvType, &drvType));
}


// Get description of a field type.

void TDEWEngine::GetFieldTypeDesc(const pcCHAR pszDriverType, const pcCHAR pszTableType, const pcCHAR pszFldType, FLDType& fldType) const
{
    TDEWException::CheckResult(DbiGetFieldTypeDesc((pCHAR) pszDriverType, (pCHAR) pszTableType, (pCHAR) pszFldType, &fldType));
}


// Get description of given idx type.

void TDEWEngine::GetIndexTypeDesc(const pcCHAR pszDriverType, const pcCHAR pszIdxType, IDXType& idxType) const
{
    TDEWException::CheckResult(DbiGetIndexTypeDesc((pCHAR) pszDriverType, (pCHAR) pszIdxType, &idxType));
}


// Get current session info.

void TDEWEngine::GetSesInfo(SESInfo& sesInfo) const
{
    TDEWException::CheckResult(DbiGetSesInfo(&sesInfo));
}


// Get system configuration.

void TDEWEngine::GetSysConfig(SYSConfig& sysConfig) const
{
    TDEWException::CheckResult(DbiGetSysConfig(&sysConfig));
}


// Get system status info.

void TDEWEngine::GetSysInfo(SYSInfo& sysInfo) const
{
    TDEWException::CheckResult(DbiGetSysInfo(&sysInfo));
}


// Get system version info.

void TDEWEngine::GetSysVersion(SYSVersion& sysVersion) const
{
    TDEWException::CheckResult(DbiGetSysVersion(&sysVersion));
}


// Get table capabilities.

void TDEWEngine::GetTableTypeDesc(const pcCHAR pszDrvType, const pcCHAR pszTableType, TBLType& tblType) const
{
    TDEWException::CheckResult(DbiGetTableTypeDesc((pCHAR) pszDrvType, (pCHAR) pszTableType, &tblType));
}


// Get list of index types.

TDEWCursor TDEWEngine::IndexTypesList(const pcCHAR pszDriverType) const
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenIndexTypesList((pCHAR) pszDriverType, &hCursor));

    return TDEWCursor(hCursor);
}


// Get a list of lang drivers.

TDEWCursor TDEWEngine::LdList() const
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenLdList(&hCursor));

    return TDEWCursor(hCursor);
}


// Get lang driver name.

string TDEWEngine::LdName(const pcCHAR pszDriver, const pcCHAR pObjName) const
{
    DBINAME name;

    TDEWException::CheckResult(DbiGetLdName((pCHAR) pszDriver, (pCHAR) pObjName, name));

    return string(name);
}


// Load a given driver.

void TDEWEngine::LoadDriver(const pcCHAR pszDrvType)
{
    TDEWException::CheckResult(DbiLoadDriver((pCHAR) pszDrvType));
}


// Get network username.

string TDEWEngine::NetUserName() const
{
    DBIUSERNAME netUserName;

    TDEWException::CheckResult(DbiGetNetUserName(netUserName));

    return string(netUserName);
}


// Get current number format.

const FMTNumber& TDEWEngine::NumberFormat() const
{
    return fFmtNumber;
}


// Open the engine.

void TDEWEngine::Open(const pcDBIEnv pEnv)
{
    TDEWException::CheckResult(DbiInit((pDBIEnv) pEnv));

    TDEWException::CheckResult(DbiGetDateFormat(&fFmtDate));
    TDEWException::CheckResult(DbiGetTimeFormat(&fFmtTime));
    TDEWException::CheckResult(DbiGetNumberFormat(&fFmtNumber));
}


// Register callback function.

void TDEWEngine::RegisterCallBack(const CBType ecbType, const UINT32 iClientData, const UINT16 iCbBufLen, const pVOID pCbBuf, const pfDBICallBack pfCb)
{
    TDEWException::CheckResult(DbiRegisterCallBack(NULL, ecbType, iClientData, iCbBufLen, pCbBuf, pfCb));
}


// Set the current session.

void TDEWEngine::SetCurrSession(TDEWSession& session)
{
    TDEWException::CheckResult(DbiSetCurrSession(session.Handle()));
}


// Set current date format.

void TDEWEngine::SetDateFormat(const FMTDate& format)
{
    fFmtDate = format;
    TDEWException::CheckResult(DbiSetDateFormat(&fFmtDate));
}


// Set lock wait time.

void TDEWEngine::SetLockRetry(const INT16 iWait)
{
    TDEWException::CheckResult(DbiSetLockRetry(iWait));
}


// Set current number format.

void TDEWEngine::SetNumberFormat(const FMTNumber& format)
{
    fFmtNumber = format;
    TDEWException::CheckResult(DbiSetNumberFormat(&fFmtNumber));
}


// Set private directory for session.

void TDEWEngine::SetPrivateDir(const pcCHAR pszDir)
{
    TDEWException::CheckResult(DbiSetPrivateDir((pCHAR) pszDir));
}


// Set current time format.

void TDEWEngine::SetTimeFormat(const FMTTime& format)
{
    fFmtTime = format;
    TDEWException::CheckResult(DbiSetTimeFormat(&fFmtTime));
}


// Get a list of table types.

TDEWCursor TDEWEngine::TableTypesList(const pcCHAR pszDriverType)
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenTableTypesList((pCHAR) pszDriverType, &hCursor));

    return TDEWCursor(hCursor);
}


// Get current time format.

const FMTTime& TDEWEngine::TimeFormat() const
{
    return fFmtTime;
}


// Use idle time.

void TDEWEngine::UseIdleTime()
{
    TDEWException::CheckResult(DbiUseIdleTime());
}


// Get a list of users loggedin.

TDEWCursor TDEWEngine::UserList() const
{
    hDBICur hCursor;

    TDEWException::CheckResult(DbiOpenUserList(&hCursor));

    return TDEWCursor(hCursor);
}
