//
//  Database Engine Wrapper
//  Version 1.0
//  EXCEPTN.CPP
//  Copyright  1994
//  B. W. Bush
//  All rights reserved
//


// Definitions.

#include <dew.h>        // database engine wrapper


// Constructor.

TDEWException::TDEWException(const DBIResult result)
  : fResult(result)
{
}


// Write the error message on a stream.

ostream& operator<<(ostream& os, const TDEWException& exception)
{
    os << exception.Message();

    return os;
}


// Answer the category.

INT16 TDEWException::Category() const
{
    return ErrCat(fResult);
}


// Answer the code.

INT16 TDEWException::Code() const
{
    return ErrCode(fResult);
}


// Answer the context.

string TDEWException::Context(const INT16 eContext) const
{
    char context[DBIMAXMSGLEN+1];

    DbiGetErrorContext(eContext, context);

    return string(context);
}


// Answer the error entry.

string TDEWException::Entry(const UINT16 uEntry, UINT32& ulNativeError) const
{
    char entry[DBIMAXMSGLEN+1];

    DbiGetErrorEntry(uEntry, &ulNativeError, entry);

    return string(entry);
}


// Answer the error information.

void TDEWException::GetInfo(DBIErrInfo& errInfo, const BOOL bFull) const
{
    DbiGetErrorInfo(bFull, &errInfo);
}


// Answer the message.

string TDEWException::Message() const
{
    char message[DBIMAXMSGLEN+1];

    DbiGetErrorString(fResult, message);

    return string(message);
}


// Answer the fResult.

DBIResult TDEWException::Result() const
{
    return fResult;
}


// Throw an exception if an error has occurred.

void TDEWException::CheckResult(const DBIResult result)
{
    switch (ErrCat(result)) {
        case ERRCAT_NONE:
            break;
        case ERRCAT_SYSTEM:
            throw TDEWSystemException(result);
        case ERRCAT_NOTFOUND:
            throw TDEWNotFoundException(result);
        case ERRCAT_DATACORRUPT:
            throw TDEWDataCorruptException(result);
        case ERRCAT_IO:
            throw TDEWIoException(result);
        case ERRCAT_LIMIT:
            throw TDEWLimitException(result);
        case ERRCAT_INTEGRITY:
            throw TDEWIntegrityException(result);
        case ERRCAT_INVALIDREQ:
            throw TDEWInvalidReqException(result);
        case ERRCAT_LOCKCONFLICT:
            throw TDEWLockConflictException(result);
        case ERRCAT_SECURITY:
            throw TDEWSecurityException(result);
        case ERRCAT_INVALIDCONTEXT:
            throw TDEWInvalidContextException(result);
        case ERRCAT_OS:
            throw TDEWOsException(result);
        case ERRCAT_NETWORK:
            throw TDEWNetworkException(result);
        case ERRCAT_OPTPARAM:
            throw TDEWOptParamException(result);
        case ERRCAT_QUERY:
            throw TDEWQueryException(result);
        case ERRCAT_VERSION:
            throw TDEWVersionException(result);
        case ERRCAT_CAPABILITY:
            throw TDEWCapabilityException(result);
        case ERRCAT_CONFIG:
            throw TDEWConfigException(result);
        case ERRCAT_WARNING:
            throw TDEWWarningException(result);
        case ERRCAT_OTHER:
            throw TDEWOtherException(result);
        case ERRCAT_COMPATIBILITY:
            throw TDEWCompatibilityException(result);
        case ERRCAT_DRIVER:
            throw TDEWDriverException(result);
        case ERRCAT_RC:
            throw TDEWInternalException(result);
        default:
            throw TDEWException(result);
    }
}
