//
//  Database Engine Wrapper
//  Version 1.0
//  FILTER.CPP
//  Copyright  1994
//  B. W. Bush
//  All rights reserved
//


// Definitions.

#include <dew.h>        // database engine wrapper


// Constructors.

TDEWFilter::TDEWFilter()
  : fHCursor(NULL),
    fHFilter(NULL)
{
}

TDEWFilter::TDEWFilter(TDEWCursor& cursor)
  : fHCursor(cursor.Handle()),
    fHFilter(NULL)
{
}

TDEWFilter::TDEWFilter(TDEWCursor& cursor, const pCANExpr pCanExpr)
  : fHCursor(cursor.Handle()),
    fHFilter(NULL)
{
    Open(pCanExpr);
}


TDEWFilter::TDEWFilter(const TDEWFilter& filter)
  : fHCursor(NULL),
    fHFilter(NULL)
{
    Clone(filter);
}


// Destructor.

TDEWFilter::~TDEWFilter()
{
    if (IsOpen())
        Close();
}


// Assignment operator.

TDEWFilter& TDEWFilter::operator=(const TDEWFilter& filter)
{
    if (IsOpen())
        Close();

    Clone(filter);

    return *this;
}


// Activate a filter.

void TDEWFilter::Activate()
{
    TDEWException::CheckResult(DbiActivateFilter(fHCursor, fHFilter));
}


// Clone a filter.

void TDEWFilter::Clone(const TDEWFilter& filter)
{
    fHCursor = filter.fHCursor;

    FILTERInfo info;
    filter.GetInfo(info);

    Open((pCANExpr) info.pCanExpr);
}


// Drop a filter.

void TDEWFilter::Close()
{
    TDEWException::CheckResult(DbiDropFilter(fHCursor, fHFilter));
    fHFilter = NULL;
}


// Deactivate a filter.

void TDEWFilter::Deactivate()
{
    TDEWException::CheckResult(DbiDeactivateFilter(fHCursor, fHFilter));
}


// Get filter information.

void TDEWFilter::GetInfo(FILTERInfo& filterInfo, const UINT16 iFilterId, const UINT16 iFilterSeqNo) const
{
    TDEWException::CheckResult(DbiGetFilterInfo(fHCursor, fHFilter, iFilterId, iFilterSeqNo, &filterInfo));
}


// Add a filter to the cursor.

void TDEWFilter::Open(TDEWCursor& cursor, const pCANExpr pCanExpr)
{
    if (IsOpen())
        Close();

    fHCursor = cursor.Handle();
    TDEWException::CheckResult(DbiAddFilter(fHCursor, 0, 1, FALSE, pCanExpr, NULL, &fHFilter));
}

void TDEWFilter::Open(const pCANExpr pCanExpr)
{
    if (IsOpen())
        Close();

    TDEWException::CheckResult(DbiAddFilter(fHCursor, 0, 1, FALSE, pCanExpr, NULL, &fHFilter));
}
