//
//  Database Engine Wrapper
//  Version 1.0
//  TRANSACT.CPP
//  Copyright  1994
//  B. W. Bush
//  All rights reserved
//


// Definitions.

#include <dew.h>        // database engine wrapper


// Constructors.

TDEWTransaction::TDEWTransaction()
  : fHDatabase(NULL),
    fHTransaction(NULL)
{
}

TDEWTransaction::TDEWTransaction(TDEWDatabase& database)
  : fHDatabase(database.Handle()),
    fHTransaction(NULL)
{
}

TDEWTransaction::TDEWTransaction(TDEWDatabase& database, const eXILType eXIL)
  : fHDatabase(database.Handle()),
    fHTransaction(NULL)
{
    Open(eXIL);
}


// Destructor.

TDEWTransaction::~TDEWTransaction()
{
    if (IsOpen())
        Close();
}


// Close transaction.

void TDEWTransaction::Close(const eXEnd eEnd)
{
    TDEWException::CheckResult(DbiEndTran(fHDatabase, fHTransaction, eEnd));
    fHTransaction = NULL;
}


// Get transaction info.

void TDEWTransaction::GetTranInfo(XInfo& xInfo) const
{
    TDEWException::CheckResult(DbiGetTranInfo(fHDatabase, fHTransaction, &xInfo));
}


// Begin a transaction.

void TDEWTransaction::Open(TDEWDatabase& database, const eXILType eXIL)
{
    if (IsOpen())
        Close();

    fHDatabase = database.Handle();

    TDEWException::CheckResult(DbiBeginTran(fHDatabase, eXIL, &fHTransaction));
}

void TDEWTransaction::Open(const eXILType eXIL)
{
    if (IsOpen())
        Close();

    TDEWException::CheckResult(DbiBeginTran(fHDatabase, eXIL, &fHTransaction));
}
